/*
FRODO: a FRamework for Open/Distributed Optimization
Copyright (C) 2008, 2009, 2010, 2011  Thomas Leaute, Brammert Ottens & Radoslaw Szymanek

FRODO is free software: you can redistribute it and/or modify
it under the terms of the GNU Affero General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

FRODO is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU Affero General Public License for more details.

You should have received a copy of the GNU Affero General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.


How to contact the authors: 
<http://lia.epfl.ch/>

EPFL / IC / IIF / LIA
Batiment IN 
Station 14 
CH - 1015 Lausanne 
Switzerland
*/

/** Contains classes used for communication between agents running within the same JVM */
package ch.epfl.lia.frodo.communication.sharedMemory;

import java.io.Serializable;

import ch.epfl.lia.frodo.communication.MessageWrapper;
import ch.epfl.lia.frodo.communication.Queue;
import ch.epfl.lia.frodo.communication.QueueInputPipeInterface;
import ch.epfl.lia.frodo.communication.QueueOutputPipeInterface;

/** Input-output pipe between two queues
 * @author Thomas Leaute 
 */
public class QueueIOPipe implements QueueInputPipeInterface, QueueOutputPipeInterface, Serializable {
	
	/** Used for serialization */
	private static final long serialVersionUID = -4232985879625166953L;
	
	/** This input pipe's queue */
	private Queue queue;
	
	/** Whether this pipe is between an agent and itself */
	private final boolean virtual;
	
	/** Constructor. Notifies the queue. 
	 * @param queue the queue to which messages should be passed
	 */
	public QueueIOPipe (Queue queue) {
		this.queue = queue;
		this.virtual = false;
		queue.toBeClosedUponKill(this);
	}
	
	/** Constructor. Notifies the queue. 
	 * @param queue 	the queue to which messages should be passed
	 * @param virtual 	whether this pipe is between an agent and itself
	 */
	public QueueIOPipe (Queue queue, boolean virtual) {
		this.queue = queue;
		this.virtual = virtual;
		queue.toBeClosedUponKill(this);
	}
	
	/**
	 * Passes a message to this pipe's queue
	 * @see ch.epfl.lia.frodo.communication.QueueOutputPipeInterface#pushMessage(ch.epfl.lia.frodo.communication.MessageWrapper)
	 */
	public void pushMessage(MessageWrapper msg) {
		
		if (! this.virtual) 
			msg.getMessage().fakeSerialize();
		
		queue.addToInbox(msg);
	}

	/** Does nothing
	 * @see ch.epfl.lia.frodo.communication.QueueInputPipeInterface#close()
	 */
	public void close() { }

	/** @see ch.epfl.lia.frodo.communication.QueueInputPipeInterface#toDOT() */
	public String toDOT() {
		return "QueueIOPipe" + this.hashCode();
	}

}
